@@ -12,7 +12,7 @@ extern crate chrono;
 use std::error::Error;
 use nom::{IResult};
 
-pub use self::parser::{Patch, File, FileMetadata, Range, Line};
+pub use self::parser::{Patch, File, FileMetadata, Range, Hunk, Line};
 use self::parser::{patch};
 
 mod parser;
@@ -45,9 +45,7 @@ pub fn parse(diff: &str) -> Result<Patch, PatchError> {
     match patch(diff.as_bytes()) {
         IResult::Done(_, p) =>
             Ok(p),
-        IResult::Incomplete(_) =>
-            Err(PatchError::ParseError),
-        IResult::Error(_) =>
+        IResult::Incomplete(_) | IResult::Error(_) =>
             Err(PatchError::ParseError),
     }
 }
